﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MyDownloader.Core;
using System.Drawing.Drawing2D;
using System.IO;

namespace minidownloader
{public delegate void MouseOverItemDel(object sender,int index);
    public partial class VideoListView : ListBox
    {
        public event MouseOverItemDel MouseOverItem;
        public Form1 parent;
        Preview preview = new Preview();
        delegate void ActionDownloader( Downloader downloader);
        Brush backBrush;
        Brush selectionBrush;
        Brush fontBrush;
        Brush progressBarBrush;
        Timer timer = new Timer();
        StringFormat centeredStringFormat=new StringFormat();
        StringFormat midleF = new StringFormat();  
        //store the latest progress of each download, to update only if changes happen
        List<double> OldProgress = new List<double>();
        List<DownloaderState> OldDownloaderState = new List<DownloaderState>();
        
        
        public VideoListView()
        {
           
            InitializeComponent();
            //Console.WriteLine("VIDEO LIST "+Environment.StackTrace);
            this.SetStyle(ControlStyles.Opaque, true);
            DoubleBuffered = false ;
            DownloadManager.Instance.DownloadAdded += new EventHandler<DownloaderEventArgs>(Instance_DownloadAdded);
            DownloadManager.Instance.DownloadRemoved += new EventHandler<DownloaderEventArgs>(Instance_DownloadRemoved);
            DownloadManager.Instance.DownloadEnded += new EventHandler<DownloaderEventArgs>(Instance_DownloadEnded);

            Rectangle backrect = new Rectangle(0, 0, this.Width, this.ItemHeight);
            backBrush = new SolidBrush(BackColor);
            selectionBrush = new LinearGradientBrush(backrect, Color.LightGreen, Color.DarkGreen, LinearGradientMode.Vertical);
            fontBrush = new LinearGradientBrush(backrect, Color.Black, Color.DarkBlue, LinearGradientMode.ForwardDiagonal);
            progressBarBrush = new LinearGradientBrush(new Rectangle(0,0,15,15), Color.Brown, Color.Orange, LinearGradientMode.ForwardDiagonal);
            for (int i = 0; i < MyDownloader.Core.DownloadManager.Instance.Downloads.Count; i++)
            {
                AddDownload(MyDownloader.Core.DownloadManager.Instance.Downloads[i]);

            }

            centeredStringFormat.Alignment = StringAlignment.Center;
            centeredStringFormat.LineAlignment = StringAlignment.Center;
            midleF.LineAlignment = StringAlignment.Center;
            timer.Tick += new EventHandler(timerTick);
            timer.Interval = 1000;
            timer.Start();
            this.MouseOverItem += new MouseOverItemDel(this.OnMouseOverItem);
        }

        protected override void OnPaint(PaintEventArgs pe)
        {
            base.OnPaint(pe);
        }
        protected override void OnPaintBackground(PaintEventArgs pevent)
        {
            //base.OnPaintBackground(pevent);
        }
        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            Bitmap bmp = new Bitmap(e.Bounds.Width, e.Bounds.Height);
            Graphics myGraphics = Graphics.FromImage(bmp);

            if (e.Index < 0) return;
           // if (!(NeedsUpdate(e.Index)||drawrequired)) return;
           if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
               myGraphics.FillRectangle(selectionBrush, e.Bounds);
            else
               myGraphics.FillRectangle(backBrush, e.Bounds);
           
            Downloader d=(Downloader)Items[e.Index];
            StringBuilder text=new StringBuilder(Path.GetFileName(d.LocalFile));
            //text.Append("\t "+d.State.ToString()+"\t"+String.Format("{0:0.##} kbps", d.Rate/1024.0));

            Rectangle TextRext = new Rectangle(e.Bounds.Left, e.Bounds.Top, (int)(e.Bounds.Width * 0.5), e.Bounds.Height);
            myGraphics.DrawString(text.ToString(), this.Font, fontBrush, TextRext, midleF);

            
            TextRext = new Rectangle(TextRext.Right, TextRext.Top, (int)(e.Bounds.Width * 0.25), e.Bounds.Height);
            myGraphics.DrawString(d.State.ToString(), this.Font, fontBrush, TextRext, centeredStringFormat);

            DrawProgressBar(myGraphics, e.Bounds, d.Progress);
            e.Graphics.DrawImage(bmp, 0, 0);
            myGraphics.Dispose();
            bmp.Dispose();
        }

        private void AddDownload(Downloader d)
        {
            this.Items.Add(d);
            this.OldDownloaderState.Add(d.State);
            this.OldProgress.Add(d.Progress);
            
        }

        #region stuff

        bool NeedsUpdate(int index)
        {
            Downloader d=((Downloader)Items[index]);
            if (OldProgress[index] != d.Progress || OldDownloaderState[index] != d.State)
            {
                OldDownloaderState[index] = d.State;
                OldProgress[index] = d.Progress;
                return true;
            }
            else return false;
        }
        

        public void SetBrushes(Brush backB, Brush selectB)
        {
            this.backBrush = backB;
            this.selectionBrush = selectB;
        }

        void Instance_DownloadAdded(object sender, DownloaderEventArgs e)
        {
            if (IsHandleCreated)
            {
                this.BeginInvoke((MethodInvoker)delegate() { AddDownload(e.Downloader); });
            }
            else
            {
                AddDownload(e.Downloader);
            }
        }
        void Instance_DownloadEnded(object sender, DownloaderEventArgs e)
        {
            //do nothing
        }
        void Instance_DownloadRemoved(object sender, DownloaderEventArgs e)
        {
            this.BeginInvoke((MethodInvoker)delegate()
            {
                Items.Remove(e.Downloader);
            });
        }
        void timerTick(object sender, EventArgs e)
        {
            if (Items.Count == 0) return;
            
            for (int i = 0; i < Items.Count; i++)
            {
                if (NeedsUpdate(i))
                {
                    Invalidate();
                    return;
                }
            }
        }


        private void DownloadsAction(ActionDownloader action)
        {
            if (SelectedItems.Count > 0)
            {
                try
                {
                    BeginUpdate();

                    this.SelectedIndexChanged -= new EventHandler(this.VideoListView_SelectedIndexChanged);

                    for (int i = SelectedIndices.Count - 1; i >= 0; i--)
                    {
                        Downloader d= (Downloader) Items[SelectedIndices[i]];
                        action(d);
                    }

                    this.SelectedIndexChanged += new EventHandler(this.VideoListView_SelectedIndexChanged);
                    this.VideoListView_SelectedIndexChanged(null, null);
                }
                finally
                {
                    EndUpdate();
                    
                }
            }
        }

        public void Pause()
        {
            DownloadsAction(
                delegate(Downloader d)
                {
                    d.Pause();
                }
            );
        }

        public void StartSelections()
        {
            DownloadsAction(
                delegate(Downloader d )
                {
                    d.Start();
                }
            );
        }

        public void PauseAll()
        {
            DownloadManager.Instance.PauseAll();
        }

        public void RemoveSelections()
        {
            if (MessageBox.Show("Are you sure that you want to remove selected downloads?",
                "Attentio!!!n", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                try
                {
                    this.SelectedIndexChanged -= new EventHandler(this.VideoListView_SelectedIndexChanged);
                    DownloadManager.Instance.DownloadRemoved -= new EventHandler<DownloaderEventArgs>(Instance_DownloadRemoved);

                    DownloadsAction(
                        delegate(Downloader d)
                        {
                            Items.Remove(d);
                            DownloadManager.Instance.RemoveDownload(d);
                        }
                    );
                }
                finally
                {
                    this.SelectedIndexChanged += new EventHandler(this.VideoListView_SelectedIndexChanged);
                    this.VideoListView_SelectedIndexChanged(null, null);

                    DownloadManager.Instance.DownloadRemoved += new EventHandler<DownloaderEventArgs>(Instance_DownloadRemoved);
                }
            }
        }
        #endregion

        private void VideoListView_SelectedIndexChanged(object sender, EventArgs e)
        {

        }



        public void RemoveCompleted()
        {
            this.BeginUpdate();
            try
            {
                DownloadManager.Instance.ClearEnded();
            }
            finally
            {
                this.EndUpdate();
            }
        }


        void DrawProgressBar(Graphics g,Rectangle bigRect, double progress)
        {
            
            int Left =bigRect.Right-bigRect.Width  / 4 -6;
            int width = bigRect.Width / 4;
            
            Rectangle smallRectangle = new Rectangle(Left, bigRect.Top + 3, width, bigRect.Height - 6);
            Bitmap b = new Bitmap(smallRectangle.Width, smallRectangle.Height);
            Graphics bmpGraphics = Graphics.FromImage(b);
            bmpGraphics.FillRectangle(Brushes.White, 0, 0, b.Width, b.Height);
            bmpGraphics.DrawRectangle(Pens.DarkBlue, 0, 0, b.Width, b.Height);
            Rectangle progressRectangle = new Rectangle(0,6, (int)(b.Width * (progress / 100)), b.Height - 12);
            bmpGraphics.FillRectangle(this.progressBarBrush, progressRectangle);
            bmpGraphics.DrawString(((int)progress).ToString() + "%", this.Font, fontBrush, new Rectangle(0,0,b.Width,b.Height), centeredStringFormat);
            bmpGraphics.Dispose();
            g.DrawImage(b, smallRectangle);
            b.Dispose();
        }

        protected override void OnSelectedIndexChanged(EventArgs e)
        {
            base.OnSelectedIndexChanged(e);
           // Cursor.Position =this.PointToScreen( new Point(Location.X + Width / 2, Location.Y + Height/2));
            ShowPreview(this.SelectedIndex);
        }
        
        //void ShowPreview()
        //{
        //    if (this.Items.Count == 0) return;
        //    Downloader d = (Downloader)GetItemUnderMouse(Cursor.Position);
        //    if (d == null) return;
        //    string thumbPath = d.LocalFile + ".jpg";
        //    string title = Path.GetFileNameWithoutExtension(d.LocalFile);
        //    string body = d.State.ToString();
        //    preview.ShowPreview(thumbPath, title, body, Cursor.Position);
        //}

        void ShowPreview(int index)
        {
            if (index < 0 || index > Items.Count - 1) return;
            Downloader d = (Downloader)Items[index];
            if (d == null) return;
            string thumbPath = d.LocalFile + ".jpg";
            string title = Path.GetFileNameWithoutExtension(d.LocalFile);
            string body = d.State.ToString();
            if (System.IO.File.Exists(thumbPath))
            {
                try
                {
                    parent.pictureBox1.Image = new Bitmap(thumbPath);
                }
                catch (Exception x) { Console.WriteLine("eror displayng thumbnail " + thumbPath + "  " + x.Message); }
            }
           
        }
      
        Downloader GetItemUnderMouse(Point mpos)
        {
            int mouseP = mpos.Y -38-this.parent.Location.Y- this.Location.Y; ;
            int aTop = this.ClientRectangle.Top;
            for (int i = TopIndex; i < Items.Count; i++)
            {
                int aBottom = ItemHeight + aTop; // cast required here
                if (aBottom>= mouseP)
                    return (Downloader)Items[i];
                aTop = aBottom;
            }
            return null;
        }

        int GetIndexItemUnderMouse(Point mpos)
        {
            int mouseP = mpos.Y;
            int aTop = this.ClientRectangle.Top;
            for (int i = TopIndex; i < Items.Count; i++)
            {
                int aBottom = ItemHeight + aTop; // cast required here
                if (aBottom >= mouseP)
                    return i;
                aTop = aBottom;
            }
            return -1;
        }

      
        public void OnMouseOverItem(object sender, int index)
        {
            if (index < 0) return;

            ShowPreview(index);
        }

        void ShowThumbnail(Image img)
        {
            parent.pictureBox1.Image = img;

        }
        //protected override void OnMouseEnter(EventArgs e)
        //{
        //    base.OnMouseEnter(e);
        //    Point mouseP =this.PointToClient(Cursor.Position);
        //    Point corner = this.Location;
        //    int index = GetIndexItemUnderMouse(new Point(mouseP.X-corner.X,mouseP.Y-corner.Y));
        //    if (index != -1)
        //        OnMouseOverItem(this, index);
        //}
    }
}
